#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_utils.zodb import volatile_property

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.features.checker.interfaces import IContentChecker, MISSING_VALUE, MISSING_LANG_VALUE, ERROR_VALUE
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_content.shared.logo.interfaces import ILogosParagraph, LOGOS_PARAGRAPH_TYPE, LOGOS_PARAGRAPH_RENDERERS, \
    LOGOS_PARAGRAPH_NAME
from pyams_i18n.interfaces import II18nManager, INegotiator, II18n
from pyams_workflow.interfaces import IWorkflow, IWorkflowState

# import packages
from pyams_content.component.paragraph import BaseParagraph, IParagraphFactory, BaseParagraphFactory, \
    BaseParagraphContentChecker
from pyams_sequence.reference import get_reference_target
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import utility_config, get_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_content import _


@implementer(ILogosParagraph)
class LogosParagraph(BaseParagraph):
    """Logos paragraph"""

    icon_class = 'fa-th-large'
    icon_hint = LOGOS_PARAGRAPH_NAME

    references = FieldProperty(ILogosParagraph['references'])
    renderer = FieldProperty(ILogosParagraph['renderer'])

    def get_targets(self, state=None, with_reference=False):
        for reference in self.references or ():
            if with_reference:
                yield reference, get_reference_target(reference, state)
            else:
                yield get_reference_target(reference, state)


@utility_config(name=LOGOS_PARAGRAPH_TYPE, provides=IParagraphFactory)
class LogosParagraphFactory(BaseParagraphFactory):
    """Logos paragraph factory"""

    name = LOGOS_PARAGRAPH_NAME
    content_type = LogosParagraph
    secondary_menu = True


@adapter_config(context=ILogosParagraph, provides=IContentChecker)
class LogosParagraphContentChecker(BaseParagraphContentChecker):
    """Logos paragraph content checker"""

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        manager = get_parent(self.context, II18nManager)
        if manager is not None:
            langs = manager.get_languages()
        else:
            negotiator = get_utility(INegotiator)
            langs = (negotiator.server_language, )
        i18n = II18n(self.context)
        for lang in langs:
            value = i18n.get_attribute('title', lang, request)
            if not value:
                field_title = translate(ILogosParagraph['title'].title)
                if len(langs) == 1:
                    output.append(translate(MISSING_VALUE).format(field=field_title))
                else:
                    output.append(translate(MISSING_LANG_VALUE).format(field=field_title, lang=lang))
        field_title = translate(ILogosParagraph['references'].title)
        if not self.context.references:
            output.append(translate(ERROR_VALUE).format(
                field=field_title,
                message=translate(_("no selected logo"))))
        else:
            for reference, target in self.context.get_targets(with_reference=True):
                if target is None:
                    output.append(translate(ERROR_VALUE).format(
                        field=field_title,
                        message=translate(_("logo '{0}' can't be found")).format(reference)))
                else:
                    workflow = IWorkflow(target, None)
                    if workflow is not None:
                        workflow_state = IWorkflowState(target)
                        if workflow_state.state not in workflow.published_states:
                            output.append(translate(ERROR_VALUE).format(
                                field=field_title,
                                message=translate(_("logo '{0}' is not published")).format(
                                    II18n(target).query_attribute('title', request=request))))
        return output


@vocabulary_config(name=LOGOS_PARAGRAPH_RENDERERS)
class LogosParagraphRendererVocabulary(SimpleVocabulary):
    """Logos paragraph renderers vocabulary"""

    def __init__(self, context=None):
        request = check_request()
        translate = request.localizer.translate
        registry = request.registry
        if not ILogosParagraph.providedBy(context):
            context = LogosParagraph()
        terms = [SimpleTerm(name, title=translate(adapter.label))
                 for name, adapter in sorted(registry.getAdapters((context, request), IContentRenderer),
                                             key=lambda x: x[1].weight)]
        super(LogosParagraphRendererVocabulary, self).__init__(terms)
