#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_content.component.theme import ICollectionsInfo

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from hypatia.interfaces import ICatalog
from pyams_content.component.theme.interfaces import ITagsInfo, IThemesInfo
from pyams_content.shared.view.interfaces import IWfView, IViewSettings, IViewThemesSettings, \
    IViewQueryParamsExtension, VIEW_THEMES_SETTINGS_KEY, IViewTagsSettings, VIEW_TAGS_SETTINGS_KEY, \
    IViewCollectionsSettings, VIEW_COLLECTIONS_SETTINGS_KEY
from zope.intid.interfaces import IIntIds

# import packages
from hypatia.query import Any
from persistent import Persistent
from pyams_catalog.query import and_
from pyams_utils.adapter import adapter_config, get_annotation_adapter, ContextAdapter
from pyams_utils.registry import get_utility
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


#
# Tags management
#

@implementer(IViewTagsSettings)
class ViewTagsSettings(Persistent, Contained):
    """View tags settings"""

    select_context_tags = FieldProperty(IViewTagsSettings['select_context_tags'])
    tags = FieldProperty(IViewTagsSettings['tags'])

    @property
    def is_using_context(self):
        return self.select_context_tags

    def get_tags(self, context):
        tags = set()
        if self.select_context_tags:
            tags_info = ITagsInfo(context, None)
            if tags_info is not None:
                tags |= set(tags_info.tags or ())
        if self.tags:
            tags |= set(self.tags)
        return tags

    def get_tags_index(self, context):
        intids = get_utility(IIntIds)
        return [intids.register(term) for term in self.get_tags(context)]


@adapter_config(context=IWfView, provides=IViewTagsSettings)
@adapter_config(name='tags', context=IWfView, provides=IViewSettings)
def view_tags_settings_factory(view):
    """View tags settings factory"""
    return get_annotation_adapter(view, VIEW_TAGS_SETTINGS_KEY, ViewTagsSettings,
                                  name='++view:tags++')


@adapter_config(name='tags', context=IWfView, provides=IViewQueryParamsExtension)
class ViewTagsQueryParamsExtension(ContextAdapter):
    """View tags query params extension"""

    weight = 50

    def get_params(self, context):
        catalog = get_utility(ICatalog)
        settings = IViewTagsSettings(self.context)
        params = None
        # check tags
        tags = settings.get_tags_index(context)
        if tags:
            params = and_(params, Any(catalog['tags'], tags))
        return params


#
# Themes management
#

@implementer(IViewThemesSettings)
class ViewThemesSettings(Persistent, Contained):
    """View themes settings"""

    select_context_themes = FieldProperty(IViewThemesSettings['select_context_themes'])
    themes = FieldProperty(IViewThemesSettings['themes'])

    @property
    def is_using_context(self):
        return self.select_context_themes

    def get_themes(self, context):
        themes = set()
        if self.select_context_themes:
            themes_info = IThemesInfo(context, None)
            if themes_info is not None:
                themes |= set(themes_info.themes or ())
        if self.themes:
            themes |= set(self.themes)
        return themes

    def get_themes_index(self, context):
        intids = get_utility(IIntIds)
        return [intids.register(term) for term in self.get_themes(context)]


@adapter_config(context=IWfView, provides=IViewThemesSettings)
@adapter_config(name='themes', context=IWfView, provides=IViewSettings)
def view_themes_settings_factory(view):
    """View themes settings factory"""
    return get_annotation_adapter(view, VIEW_THEMES_SETTINGS_KEY, ViewThemesSettings,
                                  name='++view:themes++')


@adapter_config(name='themes', context=IWfView, provides=IViewQueryParamsExtension)
class ViewThemesQueryParamsExtension(ContextAdapter):
    """View themes query params extension"""

    weight = 52

    def get_params(self, context):
        catalog = get_utility(ICatalog)
        settings = IViewThemesSettings(self.context)
        params = None
        # check themes
        themes = settings.get_themes_index(context)
        if themes:
            params = and_(params, Any(catalog['themes'], themes))
        return params


#
# Collections management
#

@implementer(IViewCollectionsSettings)
class ViewCollectionsSettings(Persistent, Contained):
    """View collections settings"""

    select_context_collections = FieldProperty(IViewCollectionsSettings['select_context_collections'])
    collections = FieldProperty(IViewCollectionsSettings['collections'])

    @property
    def is_using_context(self):
        return self.select_context_collections

    def get_collections(self, context):
        collections = set()
        if self.select_context_collections:
            collections_info = ICollectionsInfo(context, None)
            if collections_info is not None:
                collections |= set(collections_info.collections or ())
        if self.collections:
            collections |= set(self.collections)
        return collections

    def get_collections_index(self, context):
        intids = get_utility(IIntIds)
        return [intids.register(term) for term in self.get_collections(context)]


@adapter_config(context=IWfView, provides=IViewCollectionsSettings)
@adapter_config(name='collections', context=IWfView, provides=IViewSettings)
def view_collections_settings_factory(view):
    """View collections settings factory"""
    return get_annotation_adapter(view, VIEW_COLLECTIONS_SETTINGS_KEY, ViewCollectionsSettings,
                                  name='++view:collections++')


@adapter_config(name='collections', context=IWfView, provides=IViewQueryParamsExtension)
class ViewCollectionsQueryParamsExtension(ContextAdapter):
    """View collections query params extension"""

    weight = 54

    def get_params(self, context):
        catalog = get_utility(ICatalog)
        settings = IViewCollectionsSettings(self.context)
        params = None
        # check collections
        collections = settings.get_collections_index(context)
        if collections:
            params = and_(params, Any(catalog['collections'], collections))
        return params
